@@ -6,7 +6,7 @@ pub struct Cursor<T> {
     pub offset: usize,
 }
 
-impl<T: AsMut<[u8]>> Cursor<T> {
+impl<T: Bytes> Cursor<T> {
     /// Writes a slice into the data view.
     ///
     /// # Examples
@@ -14,25 +14,19 @@ impl<T: AsMut<[u8]>> Cursor<T> {
     /// ```
     /// use bin_layout::{Cursor, ErrorKind};
     ///
-    /// let mut view = Cursor::new([0; 3]);
+    /// let mut view = Cursor::new(vec![0; 3]);
     ///
-    /// assert_eq!(view.write_slice([4, 2]), Ok(()));
-    /// assert_eq!(view.write_slice([1, 2, 3]), Err(ErrorKind::InsufficientBytes));
-    /// 
-    /// assert_eq!(view.data, [4, 2, 0]);
+    /// view.write_slice([4, 2]);
     /// assert_eq!(view.offset, 2);
     /// ```
+    /// 
+    /// # Panics
+    /// This function may panic, if the data is slice `&mut [u8]`, and has not enough capacity.
+    ///
+    /// But If the data is vector `Vec<u8>`, then it may reserve extra capacity if necessary.
     #[inline]
-    pub fn write_slice(&mut self, slice: impl AsRef<[u8]>) -> Result<()> {
-        let data = self.data.as_mut();
-        let src = slice.as_ref();
-        let count = src.len();
-        unsafe {
-            let dst = data.as_mut_ptr().add(self.offset);
-            ret_err_or_add!{ (self.offset; + count) > data.len() };            
-            ptr::copy_nonoverlapping(src.as_ptr(), dst, count);
-        }
-        Ok(())
+    pub fn write_slice(&mut self, slice: impl AsRef<[u8]>) {
+        self.offset = self.data.write_slice_at(self.offset, slice);
     }
 }
 
@@ -69,7 +63,11 @@ impl<'de> Cursor<&'de [u8]> {
     #[inline]
     pub fn read_slice(&mut self, len: usize) -> Result<&'de [u8]> {
         let total_len = self.offset + len;
-        let slice = self.data.get(self.offset..total_len).ok_or(InsufficientBytes)?;
+        let slice = self
+            .data
+            .get(self.offset..total_len)
+            .ok_or(InsufficientBytes)?;
+
         self.offset = total_len;
         Ok(slice)
     }
@@ -77,11 +75,8 @@ impl<'de> Cursor<&'de [u8]> {
 
 impl<T> Cursor<T> {
     #[inline]
-    pub const fn new(data:T) -> Self {
-        Self {
-            data,
-            offset: 0,
-        }
+    pub const fn new(data: T) -> Self {
+        Self { data, offset: 0 }
     }
 }
 
@@ -91,4 +86,3 @@ impl<T> From<T> for Cursor<T> {
         Self::new(data)
     }
 }
-
